#ifndef AUBO_SDK_AUBO_API_INTERFACE_H
#define AUBO_SDK_AUBO_API_INTERFACE_H

#include <aubo/system_info.h>
#include <aubo/runtime_machine.h>
#include <aubo/register_control.h>
#include <aubo/robot_interface.h>
#include <aubo/global_config.h>
#include <aubo/math.h>
#include <aubo/socket.h>
#include <aubo/serial.h>

namespace arcs {
namespace common_interface {

/**
 * ARCS 机器人及外部轴等控制API接口
 */
class ARCS_ABI_EXPORT AuboApi
{
public:
    AuboApi();
    virtual ~AuboApi();

    /// 获取纯数学相关接口
    MathPtr getMath();

    /// 获取系统信息
    SystemInfoPtr getSystemInfo();

    /// 获取运行时接口
    RuntimeMachinePtr getRuntimeMachine();

    /// 对外寄存器接口
    RegisterControlPtr getRegisterControl();

    /// 获取机器人列表
    std::vector<std::string> getRobotNames();

    /// 根据名字获取 RobotInterfacePtr 接口
    RobotInterfacePtr getRobotInterface(const std::string &name);

    /// 获取外部轴接口

    /// 获取独立 IO 模块接口

    SocketPtr getSocket();
    SerialPtr getSerial();

protected:
    void *d_{ nullptr };
};
using AuboApiPtr = std::shared_ptr<AuboApi>;

// clang-format off
#define AuboApi_DECLARES                       \
    _FUNC(AuboApi, 0, getMath)                 \
    _FUNC(AuboApi, 0, getSystemInfo)           \
    _FUNC(AuboApi, 0, getRuntimeMachine)       \
    _FUNC(AuboApi, 0, getRegisterControl)      \
    _FUNC(AuboApi, 0, getRobotNames)           \
    _FUNC(AuboApi, 1, getRobotInterface, name) \
    _FUNC(AuboApi, 0, getSocket)               \
    _FUNC(AuboApi, 0, getSerial)
// clang-format on

} // namespace common_interface
} // namespace arcs

#endif // AUBO_SDK_AUBO_API_H
