#ifndef AUBO_SDK_RUNTIME_MACHINE_INTERFACE_H
#define AUBO_SDK_RUNTIME_MACHINE_INTERFACE_H

#include <memory>
#include <aubo/global_config.h>
#include <aubo/type_def.h>

namespace arcs {
namespace common_interface {

/**
 * The RuntimeMachine class
 */
class ARCS_ABI_EXPORT RuntimeMachine
{
public:
    RuntimeMachine();
    virtual ~RuntimeMachine();

    /// 设置下一行脚本的运行上下文
    int setPlanContext(int tid, int lineno, const std::string &comment);

    /// 获取耗时的接口执行状态, 如 setPersistentParameters
    std::tuple<std::string, std::string> getExecutionStatus();

    /// 跳转到指定行号
    int gotoLine(int lineno);

    /// 获取当前运行上下文
    std::tuple<int, int, std::string> getPlanContext();

    /// 开始运行时
    int start();

    /// 终止运行时
    int stop();

    /// 终止脚本运行(在关节空间进行规划，以最快速度退出)
    int abort();

    /// 暂停解释器
    int pause();

    /// 单步运行
    int step();

    /// 恢复解释器
    int resume();

    /// 获取规划器的状态
    RuntimeState getStatus();

    /// 设置断点
    int setBreakPoint(int lineno);

    /// 移除断点
    int removeBreakPoint(int lineno);

    /// 清除所有断点
    int clearBreakPoints();

protected:
    void *d_;
};

using RuntimeMachinePtr = std::shared_ptr<RuntimeMachine>;

// clang-format off
#define RuntimeMachine_DECLARES                                     \
    _FUNC(RuntimeMachine, 3, setPlanContext, tid, lineno, comment)  \
    _FUNC(RuntimeMachine, 1, gotoLine, lineno)                      \
    _FUNC(RuntimeMachine, 0, getPlanContext)                        \
    _FUNC(RuntimeMachine, 0, getExecutionStatus)                    \
    _FUNC(RuntimeMachine, 0, start)                                 \
    _FUNC(RuntimeMachine, 0, stop)                                  \
    _FUNC(RuntimeMachine, 0, abort)                                 \
    _FUNC(RuntimeMachine, 0, pause)                                 \
    _FUNC(RuntimeMachine, 0, step)                                  \
    _FUNC(RuntimeMachine, 0, resume)                                \
    _FUNC(RuntimeMachine, 0, getStatus)                             \
    _FUNC(RuntimeMachine, 1, setBreakPoint, lineno)                 \
    _FUNC(RuntimeMachine, 1, removeBreakPoint, lineno)              \
    _FUNC(RuntimeMachine, 0, clearBreakPoints)
// clang-format on
} // namespace common_interface
} // namespace arcs
#endif // AUBO_SDK_RUNTIME_MACHINE_H
