import time
import numpy as np
import pyaubo_sdk

robot_ip = "127.0.0.1"
robot_port = 30004
M_PI = 3.14159265358979323846
robot_rpc_client = pyaubo_sdk.RpcClient()

def waitarrived(robot_name, target_q):
    target_q_np = np.array([target_q])
    while 1:
        current_q_np = np.array(robot_rpc_client.getRobotInterface(robot_name)
                                .getRobotState().getJointPositions())
        distance = np.linalg.norm(current_q_np - target_q_np, ord=np.Inf, axis=1)  # inf norm
        if distance < 0.0001:
            break
        time.sleep(1)

def example_movej(robot_name):
    q1 = [-2.05177, -0.400292, 1.19625, 0.0285152, 1.57033, -2.28774]
    q2 = [-0.236948, -0.465633, 1.1002, -0.00600456, 1.56824, -0.472864]
    q3 = [1.38167, -0.0115091, 1.66958, 0.107726, 1.572, 1.14576]
    robot_rpc_client.getRuntimeMachine().start()
    print("Moving to q1...")
    robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)
    waitarrived(robot_name, q1)
    print("Moving to q2...")
    robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q2, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)
    waitarrived(robot_name, q2)
    print("Moving to q3...")
    robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q3, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)
    waitarrived(robot_name, q3)
    robot_rpc_client.getRuntimeMachine().stop()

if __name__ == '__main__':
    robot_rpc_client.connect(robot_ip, robot_port)
    if robot_rpc_client.hasConnected():
        print("Robot rcp_client connected successfully!")
        robot_rpc_client.login("aubo", "123456")
        if robot_rpc_client.hasLogined():
            print("Robot rcp_client logined successfully!")
            robot_name = robot_rpc_client.getRobotNames()[0]
            example_movej(robot_name);