import time
import numpy as np
import pyaubo_sdk
import os

robot_ip = "127.0.0.1"
robot_port = 30004
M_PI = 3.14159265358979323846
robot_rpc_client = pyaubo_sdk.RpcClient()

def waitarrived(robot_name, target_q):
    target_q_np = np.array([target_q])
    while 1:
        current_q_np = np.array(robot_rpc_client.getRobotInterface(robot_name)
                                .getRobotState().getJointPositions())
        distance = np.linalg.norm(current_q_np - target_q_np, ord=np.Inf,
                                  axis=1)  # inf norm
        if distance < 0.0001:
            break
        time.sleep(0.1)

def example_movel(robot_name):
    cnt = 0

    q0 = [-2.40194, 0.103747, 1.7804,
          0.108636, 1.57129, -2.6379]
    Waypoint_0_p = [-0.155944, -0.727344, 0.439066,
                    3.05165, 0.0324355, 1.80417];
    Waypoint_1_p = [-0.581143, -0.357548, 0.439066,
                    3.05165, 0.0324355, 1.80417];
    Waypoint_2_p = [0.503502, -0.420646, 0.439066,
                    3.05165, 0.0324355, 1.80417];
    robot_rpc_client.getRuntimeMachine().start()
    print("Moving to q0...")
    robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q0, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)
    waitarrived(robot_name, q0)
    for i in range(5):
        print("Moving to p0...")
        robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
            .moveLine(Waypoint_0_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0, 0)
        while 1:
            cur = robot_rpc_client.getRobotInterface(robot_name) \
                .getRobotState().getJointPositions()
            inverse_result = robot_rpc_client.getRobotInterface(robot_name) \
                .getRobotAlgorithm().inverseKinematics(cur, Waypoint_0_p)
            if inverse_result[1] >= 0:
                inverse_value = inverse_result[0]
                cnt = 0
                break
            cnt = cnt + 1
            if cnt > 10:
                os._exit()
        waitarrived(robot_name, inverse_value)
        inverse_value.clear()

        print("Moving to p1...")
        robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
            .moveLine(Waypoint_1_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0, 0)
        while 1:
            cur = robot_rpc_client.getRobotInterface(robot_name) \
                .getRobotState().getJointPositions()
            inverse_result = robot_rpc_client.getRobotInterface(robot_name) \
                .getRobotAlgorithm().inverseKinematics(cur, Waypoint_1_p)
            if inverse_result[1] >= 0:
                inverse_value = inverse_result[0]
                cnt = 0
                break
            cnt = cnt + 1
            if cnt > 10:
                os._exit()
        waitarrived(robot_name, inverse_value)
        inverse_value.clear()

        print("Moving to p2...")
        robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
            .moveLine(Waypoint_2_p, 250 * (M_PI / 180), 1000 * (M_PI / 180), 0, 0)
        while 1:
            cur = robot_rpc_client.getRobotInterface(robot_name) \
                .getRobotState().getJointPositions()
            inverse_result = robot_rpc_client.getRobotInterface(robot_name) \
                .getRobotAlgorithm().inverseKinematics(cur, Waypoint_2_p)
            if inverse_result[1] >= 0:
                inverse_value = inverse_result[0]
                cnt = 0
                break
            cnt = cnt + 1
            if cnt > 10:
                os._exit()
        waitarrived(robot_name, inverse_value)
        inverse_value.clear()
    robot_rpc_client.getRuntimeMachine().stop()

if __name__ == '__main__':
    robot_rpc_client.connect(robot_ip, robot_port)
    if robot_rpc_client.hasConnected():
        print("Robot rcp_client connected successfully!")
        robot_rpc_client.login("aubo", "123456")
        if robot_rpc_client.hasLogined():
            print("Robot rcp_client logined successfully!")
            robot_name = robot_rpc_client.getRobotNames()[0]
            example_movel(robot_name);
